import java.io.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Date;
import java.util.Vector;
import java.util.Iterator;

/**
 * The <code>ExamSummary</code> window presents results of an exam
 * in a convenient way.
 *
 * Mod Log:
 * 02jan00      Modified for correct operation of Prev button.
 *
 * @author	JPL
 */
public class ExamSummary extends JFrame
                         implements QuestionSelectListener, NavButtonListener
{
	/**
	 * Score needed to pass the exam
	 */
	private static int PASS_SCORE = 71;

	/**
	 * List of <code>Question</code>s that were asked (and answered) during
	 * the exam
	 */
	private java.util.List questions;

	/**
	 * Cached marks for all questions. True in this table corresponds to
	 * a correctly answered question at a given index of the list
	 */
	private boolean[] marks;

	/**
	 * Cached number of questions answered correctly
	 */
	private int correctNum;

	/**
	 * Date and time of exam completion (the moment of creating this
	 * <code>ExamSummary</code> object)
	 */
	private Date timestamp;

	/**
    * Index of the currently displayed question
    */
	private int curQuestion;

	/**
	 * Cached score in percent
	 */
	private int score;

	/**
	 * Main navigation bar displaying question icons
	 */
	private QuestionsBar navBar;

	/**
	 * Question, answers & explanation display
	 */
	private QuestionDisplay qDisplay;

	/**
	 * Short info about the currently displayed question
	 */
	private JLabel qLabel;

	/**
	 * Secondary navigation bar, containing navigation buttons
	 */
	private NavButtonsBar btnsBar;

	/**
	 * Flag indicating the enabled/disabled state of the ToolTipManager
	 * when this ExamSummary was created. ExamSummary will restore this
	 * state when its window is closed.
	 */
	private boolean initialShowTips;


	/**
	 * Constructs a new <code>ExamSummary</code> based on the given list
	 * of questions from an exam.
	 */
	public ExamSummary (java.util.List questions)
	{
		super ("Exam Summary");

		initialShowTips = ToolTipManager.sharedInstance().isEnabled();
		this.questions = questions;

		timestamp = new Date();
		markQuestions();
		score = getScore();

		JPanel summary = createSummary();
		summary.setAlignmentX (Component.CENTER_ALIGNMENT);

		JPanel qLabelPanel = createQLabelPanel();
		qLabelPanel.setAlignmentX (Component.CENTER_ALIGNMENT);

		navBar = new QuestionsBar (marks, this);
		navBar.setAlignmentX (Component.CENTER_ALIGNMENT);

		qDisplay = new QuestionDisplay();
		qDisplay.setAlignmentX (Component.CENTER_ALIGNMENT);

		btnsBar = createBtnsBar();
		btnsBar.setAlignmentX (Component.CENTER_ALIGNMENT);

		JPanel contentPane = new JPanel();
		contentPane.setLayout (new BoxLayout (contentPane, BoxLayout.Y_AXIS));
		contentPane.add (summary);
		contentPane.add (navBar);
		contentPane.add (qLabelPanel);
		contentPane.add (Box.createVerticalGlue());
		contentPane.add (qDisplay);
		contentPane.add (Box.createVerticalGlue());
		contentPane.add (btnsBar);
		contentPane.setBorder (BorderFactory.createEmptyBorder (5, 5, 5, 5));
		setContentPane (contentPane);

		displayQuestion (0);

		addMenuBar();

		setDefaultCloseOperation (DISPOSE_ON_CLOSE);
		addWindowListener (new WindowAdapter()
			{
				public void windowClosing (WindowEvent e)
				{
					ToolTipManager.sharedInstance().setEnabled (
						ExamSummary.this.initialShowTips);
				}
			});
	}

	/**
	 * Invoked by <code>btnsBar</code> to indicate a button was pressed
	 */
	public void buttonClicked (String actionCommand)
	{
		if (actionCommand.equals ("NEXT"))
			displayQuestion (curQuestion + 1);
		else if (actionCommand.equals ("PREV"))
			displayQuestion (curQuestion - 1);
		else if (actionCommand.equals ("NEXT_CORRECT"))
			displayQuestion (getNextQuestion (true));
		else if (actionCommand.equals ("NEXT_INCORRECT"))
			displayQuestion (getNextQuestion (false));
		else if (actionCommand.equals ("PREV_CORRECT"))
			displayQuestion (getPrevQuestion (true));
		else if (actionCommand.equals ("PREV_INCORRECT"))
			displayQuestion (getPrevQuestion (false));
		else if (actionCommand.equals ("SAVE"))
			saveSummary();
	}

	/**
	 * Invoked by <code>navBar</code> when the user selects a question
	 */
	public void questionSelected (int index)
	{
		displayQuestion (index);
	}

	private void addMenuBar ()
	{
		JMenuBar menuBar = new JMenuBar();
		JMenu menu = new JMenu ("Preferences");
		JCheckBoxMenuItem cb =
			new JCheckBoxMenuItem (
				"Show tool tips", ToolTipManager.sharedInstance().isEnabled());
		cb.addItemListener (new ItemListener ()
			{
				public void itemStateChanged (ItemEvent e)
				{
					boolean showTips = e.getStateChange() == ItemEvent.SELECTED;
					ToolTipManager.sharedInstance().setEnabled (showTips);
				}
			});
		menu.add (cb);
		menuBar.add (menu);
		setJMenuBar (menuBar);
	}

	private NavButtonsBar createBtnsBar ()
	{
		boolean prevNextEnabled = questions.size() > 1;

		btnsBar = new NavButtonsBar (
			this,
			prevNextEnabled,
			prevNextEnabled && (score > 0),
			prevNextEnabled && (score < 100));

		return btnsBar;
	}

	private JPanel createQLabelPanel ()
	{
		JPanel qLabelPanel = new JPanel();
		qLabelPanel.setLayout (new BoxLayout (qLabelPanel, BoxLayout.X_AXIS));
		qLabel = new JLabel();
		qLabel.setHorizontalAlignment (SwingConstants.LEFT);
		qLabel.setAlignmentY (Component.TOP_ALIGNMENT);
		qLabelPanel.setBorder (BorderFactory.createCompoundBorder (
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder (2, 2, 2, 2)));
		qLabelPanel.add (qLabel);
		qLabelPanel.add (Box.createHorizontalGlue());

		return qLabelPanel;
	}

	private JPanel createSummary ()
	{
		JPanel summary = new JPanel();
		summary.setLayout (new BoxLayout (summary, BoxLayout.X_AXIS));
		JLabel l = new JLabel (
			"Your score: " + score + "% (" + correctNum + " of " +
			questions.size() + " right). " +
			((score >= PASS_SCORE)
			? "You passed!" : "You failed. Better luck next time!"));
		l.setAlignmentY (Component.TOP_ALIGNMENT);
		l.setForeground (Color.black);
		l.setToolTipText ("Date finished: " + timestamp);
		summary.add (l);
		summary.add (Box.createHorizontalGlue());

		JPanel p = new JPanel()
			{
				public Dimension getMaximumSize ()
				{
					return new Dimension (
						super.getMaximumSize().width, this.getPreferredSize().height);
				}
			};
		p.setLayout (new BoxLayout (p, BoxLayout.Y_AXIS));
		summary.setAlignmentX (Component.LEFT_ALIGNMENT);

		JComponent hr = new JComponent ()
			{
				public Dimension getPreferredSize ()
				{
					return new Dimension (super.getPreferredSize().width, 1);
				}

				public void paintComponent (Graphics g)
				{
					g.setColor (Color.black);
					g.drawLine (0, 0, getSize().width, 0);
				}
			};

		hr.setAlignmentX (Component.LEFT_ALIGNMENT);

		p.add (summary);
		p.add (Box.createVerticalStrut (1));
		p.add (hr);
		p.add (Box.createVerticalStrut (10));
		p.setBorder (BorderFactory.createEmptyBorder (2, 2, 2, 2));
		return p;
	}

	private void displayQuestion (int index)
	{
		while (index < 0) index += questions.size();
		index %= questions.size();
		curQuestion = index;
		navBar.setSelected (curQuestion);
		Question q = getQuestion (index);
		qLabel.setText (
			"Question #" + (index+1) + ", answered " +
			(q.isCorrect() ? "correctly" : "incorrectly"));
		qDisplay.setQuestion (q);
	}

	private int getNextQuestion (boolean correct)
	{
		for (int i = curQuestion+1;; i++)
		{
			i %= questions.size();
			if (getQuestion (i).isCorrect() == correct) return i;
		}
	}

	private int getPrevQuestion (boolean correct)
	{
		for (int i = curQuestion-1;; i--)
		{
			if (i < 0) i = questions.size()-1;
			if (getQuestion (i).isCorrect() == correct) return i;
		}
	}

	private Question getQuestion (int index)
	{
		return (Question) questions.get (index);
	}

	private int getScore ()
	{
       return correctNum * 100 / questions.size();
	}

	/**
	 * Initializes <code>marks</code> table and <code>correctNum</code>
	 * by marking each question on the list.
	 */
	private void markQuestions ()
	{
		marks = new boolean [questions.size()];
		int j = 0;

		for (Iterator i = questions.iterator(); i.hasNext(); j++)
		{
			Question q = (Question) i.next();
			marks[j] = q.markQuestion();
			if (marks[j]) correctNum++;
		}
	}

	/**
	 * Saves summary to a text file and displays a little confirmation dialog
	 */
	private void saveSummary ()
	{
		final String FILENAME = "Jxam-Summary.txt";
		final String nl = System.getProperty ("line.separator");

		try
		{
			java.io.PrintWriter out = new java.io.PrintWriter (
				new java.io.FileWriter (FILENAME));

			out.print (nl + "Exam Results: " + nl);
			out.print ("Date: " + timestamp + nl + nl);
			out.print (nl + nl + nl + "******** SUMMARY ********" + nl + nl);
			out.print ("Score = " + score + "%");
			out.print (nl + nl + nl);

			if (score >= PASS_SCORE)
				out.print ("C O N G R A T U L A T I O N S  -  Y O U   P A S S E D");
			else
				out.print ("Sorry, you didn't achieve a pass grade - try again");

			for (int i = 0; i < questions.size(); i++)
				out.print (getQuestion (i).toString (i));

			out.close();

			JOptionPane.showConfirmDialog (
				this,
				"Summary saved to file " + FILENAME,
				"Summary saved",
				JOptionPane.DEFAULT_OPTION,
				JOptionPane.PLAIN_MESSAGE);
		}
		catch (java.io.IOException e)
		{
			JOptionPane.showConfirmDialog (
				this,
				"An IO error occured while saving the summary to file " + FILENAME,
				"Problems!",
				JOptionPane.DEFAULT_OPTION,
				JOptionPane.ERROR_MESSAGE);
		}
	}

/*
	// THIS code has nothing to do with the rest of the class.
	// It is here for my debugging/testing convenience only

	public static void main (String[] args) throws Exception
	{
		BufferedReader r = new BufferedReader (new FileReader ("res"));
		Vector v = new Vector();
		String l;
		while ((l = r.readLine()) != null) v.addElement (l);
		r.close();
		ObjectOutputStream os = new ObjectOutputStream (
			new FileOutputStream ("help.db"));
		os.writeObject (v);
		os.close();
	}
*/
}