import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * A horizontal navigation bar showing each question as a small icon,
 * in color based on whether it is marked as correct or not.
 * <code>QuestionsBar</code> will fire <code>questionSelected</code>
 * calls on its registered <code>QuestionSelectListener</code> when
 * the user selects another question.
 *
 * @author	JPL
 */
public class QuestionsBar extends JPanel
{
	private QuestionBtn[] questions;
	private int selIndex = 0;
	private QuestionSelectListener listener;

	/**
	 * Constructs a new <code>QuestionsBar</code>, based on the given array
	 * of marks. The specified <code>QuestionSelectListener</code> will be
	 * notified as another question is selected. Initially, the first question
	 * is marked as selected.
	 */
	public QuestionsBar (boolean[] marks, QuestionSelectListener listener)
	{
		this.listener = listener;

		questions = new QuestionBtn [marks.length];
		setLayout (new BoxLayout (this, BoxLayout.Y_AXIS));

		JPanel p = new JPanel();
		p.setAlignmentX (Component.CENTER_ALIGNMENT);
		p.setLayout (new FlowLayout (FlowLayout.LEFT, 0, 0));

		for (int i = 0; i < marks.length; i++)
		{
			questions[i] = new QuestionBtn (marks[i], i);
			p.add (questions[i]);
		}

		p.setMaximumSize (p.getPreferredSize());
		add (p);
		if (selIndex != -1) questions[selIndex].setSelected (true);

		setBorder (BorderFactory.createEmptyBorder (0, 0, 5, 0));
	}

	public Dimension getMaximumSize ()
	{
		return new Dimension (
			super.getMaximumSize().width, getPreferredSize().height);
	}

	public void setSelected (int index)
	{
		if (selIndex != -1) questions[selIndex].setSelected (false);
		selIndex = index;
		questions[selIndex].setSelected (true);
	}

	private void btnClicked (int index)
	{
		if (index == selIndex) return;
		if (selIndex != -1) questions[selIndex].setSelected (false);
		selIndex = index;
		questions[selIndex].setSelected (true);
		listener.questionSelected (selIndex);
	}

	private class QuestionBtn extends JComponent
	{
		private int myIndex;
		private boolean correct, selected;

		public QuestionBtn (boolean correct, int index)
		{
			setToolTipText ("Displays question #" + (index+1));
			this.correct = correct;
			myIndex = index;

			addMouseListener (new MouseAdapter()
				{
					public void mouseClicked (MouseEvent e)
					{
						QuestionsBar.this.btnClicked (myIndex);
					}
				});
		}

		public Dimension getMinimumSize ()
		{
			return getPreferredSize();
		}

		public Dimension getPreferredSize ()
		{
			return new Dimension (12, 10);
		}

		public void paintComponent (Graphics g)
		{
			Dimension size = getSize();

			if (selected)
			{
				g.setColor (Color.black);
				g.drawRect (0, 0, size.width-1, size.height-1);
			}

			g.setColor (correct ? Color.green.darker() : Color.red.darker());
			g.fillRect (2, 2, size.width-4, size.height-4);

			Color bright = g.getColor().brighter();
			Color dark = g.getColor().darker();

			g.setColor (bright);
			g.drawLine (2, 2, size.width-3, 2);
			g.drawLine (2, 2, 2, size.height-3);
			g.setColor (dark);
			g.drawLine (2, size.height-3, size.width-3, size.height-3);
			g.drawLine (size.width-3, 2, size.width-3, size.height-3);
		}

		public void setSelected (boolean selected)
		{
			this.selected = selected;
			repaint();
		}
	}
}